@@ -19,6 +19,7 @@ function(
     for(i in 1:length(use_adducts)){if((adducts[adducts[,1]==use_adducts[i],6]!=ion_mode)){stop(paste(use_adducts[i]," not in ion mode ",ion_mode,sep=""))}};
 	if(!is.data.frame(peaklist)){stop("peaklist must be a data.frame")}
 	if(length(peaklist[1,])>3){stop("peaklist with > 3 columns not allowed")}
+	if(!length(peaklist[,1])>1){stop("peaklist with one entry - doesn`t make sense ...")}
 	if(!is.numeric(peaklist[,1])||!is.numeric(peaklist[,2])||!is.numeric(peaklist[,3])){stop("peaklist columns not numeric")}
     ############################################################################
     cat("\n(1) Combine adducts...");
@@ -50,14 +51,16 @@ function(
     cat("done.");
     ############################################################################
 	cat("\n(2) Build peaklist kd-tree, screen, ... \n");
+	inter<-as.numeric(interactive())
 	pBar <- txtProgressBar( min = 0, max = length(peaklist[,1]), style = 3 )
 	peakTree<-.Call("kdtree4", 
 		as.matrix(peaklist[,c(1,3)]),
+		as.integer(inter),
 		pBar,
 		PACKAGE="nontarget"
 	);
 	close(pBar)
-	peakTree<-peakTree[,1:4];
+	peakTree<-peakTree[,1:4,drop=FALSE];
 	if(ppm=="TRUE"){ppm2<-1}else{ppm2<-0}
 	pBar <- txtProgressBar(min = 0, max = length(peaklist[,1]), style = 3 )
  	relat<-.Call("adduct_search",
@@ -67,6 +70,7 @@ function(
 		as.numeric(mztol), 				# precision measurement mass
 		as.integer(ppm2),				# precision measurement - mass in ppm?
 		as.numeric(rttol),				# precision measurement RT
+		as.integer(inter),
 		pBar,	
 		PACKAGE="nontarget"
 	);
@@ -115,7 +119,7 @@ function(
 		}
 	}	
     ID<-seq(1:length(peaklist[,1]));	
-    list_adducts<-data.frame(peaklist[,1:3],ID,getit4,getit2,getit1,getit3);
+    list_adducts<-data.frame(peaklist[,1:3],ID,getit4,getit2,getit1,getit3,stringsAsFactors=FALSE);
     names(list_adducts)<-c("m/z","int","ret","peak ID","group ID","to ID","adduct(s)","mass tolerance");
 	# (3.2) Groupwise ##########################################################
     group1<-paste("/",as.character(1:max(groups)),"/",sep="") 	# groupnumber?
@@ -142,11 +146,11 @@ function(
 	}
 	group2<-substr(group2,2,nchar(group2))
 	group3<-substr(group3,2,nchar(group3))
-    grouping<-data.frame(group1,group2,group3,group4);
+    grouping<-data.frame(group1,group2,group3,group4,stringsAsFactors=FALSE);
     names(grouping)<-c("group ID","peak IDs","adducts","ambig.?");
 	overlaps<-sum(as.numeric(as.logical(group4)));
     # (3.3) counts of adduct matches ###########################################
-    hits<-data.frame(use_adducts,rep(0,length(use_adducts)));
+    hits<-data.frame(use_adducts,rep(0,length(use_adducts)),stringsAsFactors=FALSE);
     names(hits)<-c("names","counts");
     for(i in 1:length(group1)){
 		this1<-strsplit(as.character(group3[i]),"/")[[1]];
@@ -155,7 +159,7 @@ function(
 		};
     };
 	# list #####################################################################
-    parameters<-data.frame(rttol,mztol,ppm,ion_mode);	
+    parameters<-data.frame(rttol,mztol,ppm,ion_mode,stringsAsFactors=FALSE);	
     adduct<-list(list_adducts,parameters,grouping,hits,overlaps);
     names(adduct)<-c("adducts","Parameters","Peaks in adduct groups","Adduct counts","Overlaps");
     cat("done.\n\n");
